﻿using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

// Much class, many method, wow.
namespace Stars
{
    class AsteroidSystem
    {
        Asteroid[] asteroids;

        public void init(Texture2D tex, int length = 45)
        {
            asteroids = new Asteroid[length];

            for (int i = 0; i < asteroids.Length; i++)
            {
                asteroids[i] = new Asteroid(tex);
            }
        }

        public void update()
        {
            for (int i = 0; i < asteroids.Length; i++)
            {
                asteroids[i].update();
            }
        }

        public void draw(SpriteBatch sb)
        {
            for (int i = 0; i < asteroids.Length; i++)
            {
                asteroids[i].draw(sb);
            }
        }

        /// <summary>Gets the nearest asteroids specified by position.</summary>
        /// <param name="pos">position</param>
        /// <param name="radius">radius in pixels</param>
        /// <returns>Asteroids! :D</returns>
        public Asteroid[] getNearestAsteroids(Vector2 pos, int radius)
        {
            Vector2 min = new Vector2(pos.X - radius, pos.Y - radius);
            Vector2 max = new Vector2(pos.X + radius, pos.Y + radius);
            
            Asteroid[] toReturn = new Asteroid[asteroids.Length];
            int num = 0;

            for (int i = 0; i < asteroids.Length; i++)
            {
                if (asteroids[i].pos.X + asteroids[i].tex.Width / 2 > min.X && asteroids[i].pos.X + asteroids[i].tex.Width / 2 < max.X &&
                    asteroids[i].pos.Y + asteroids[i].tex.Height / 2 > min.Y && asteroids[i].pos.Y + asteroids[i].tex.Height / 2 < max.Y)
                //if (asteroids[i].pos.X > min.X && asteroids[i].pos.X < max.X &&
                //asteroids[i].pos.Y > min.Y && asteroids[i].pos.Y < max.Y)
                {
                    toReturn[num] = asteroids[i];
                    num++;
                }
            }

            Array.Resize(ref toReturn, num);
            return toReturn;
        }
    }

    class Asteroid
    {
        public Texture2D tex;
        public Vector2 pos;
        Rectangle rect;

        public float rot;
        float rotStep;

        public Asteroid(Texture2D texture)
        {
            this.tex = texture;

            Random ran = new Random(Guid.NewGuid().GetHashCode());

            this.rot = (float)Math.PI * ran.Next(361) / 180;
            this.rotStep = (float)(ran.Next(20, 300) / 10) / 50 / 10 / 2;
            this.pos = new Vector2(
                ran.Next(Game.map.X, Game.map.Width - tex.Width),
                ran.Next(Game.map.Y, Game.map.Height - tex.Height));
            this.rect = new Rectangle(ran.Next(5) * (tex.Width / Game.asteroidsInImage), 0, tex.Width / Game.asteroidsInImage, tex.Height);
        }

        public void update()
        {
            rot += rotStep;
        }

        public void draw(SpriteBatch sb)
        {
            sb.Draw(tex, pos, rect, Color.White, rot, new Vector2(tex.Width / Game.asteroidsInImage / 2 , tex.Height / 2), 2f, SpriteEffects.None, 0);
        }

        public bool collision(Vector2 position, Texture2D texture)
        {
            return (pos.X + rect.Width > position.X && pos.X < position.X + texture.Width && pos.Y + rect.Height > position.Y && pos.Y < position.Y + texture.Width);
        }
    }
}
