﻿using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Stars
{
    class AsteroidMiniMap
    {
        public const int width = 120;
        public const int height = 80;
        public static float nineteendegrees = MathHelper.ToRadians(90);
        byte dotWidth2 = 2; // Why? Because we have plenty of RAM for an extra byte, but we don't like calculations! WTF AM I THINKING?
        const byte offset = 14;

        public Texture2D dot;
        public Texture2D tex;
        public Vector2 pos;

        Vector2 target;
        float playerRot;

        public Vector2[] positions = new Vector2[0];
        public float[] rotations = new float[0];

        Color asteroidColor = new Color(220, 0, 0, 120);
        Color playerColor = new Color(2, 255, 90, 120);

        public AsteroidMiniMap(GraphicsDevice dev)
        {
            this.dot = Game.emptyTexture(4, 4, Color.White, dev);
            this.dotWidth2 = (byte)(dot.Width / 2);
            this.tex = Game.emptyTexture(width, height, new Color(10, 100, 220, 120), dev);
            this.pos = new Vector2(Game.screenWidth - width - offset, Game.screenHeight - height - offset);
        }

        public void update(Asteroid[] asteroids, Vector2 playerPos, Vector2 targetPos, float playerRotation)
        {
            positions = new Vector2[asteroids.Length];
            rotations = new float[asteroids.Length];

            this.playerRot = playerRotation;

            for (int i = 0; i < asteroids.Length; i++)
            {
                //positions[i] = drawPos(new Vector2((asteroids[i].pos.X / Game.map.Width) * width, (asteroids[i].pos.Y / Game.map.Height) * height));
                positions[i] = asteroidDrawPos(new Vector2(
                    ((asteroids[i].pos.X + asteroids[i].tex.Width / 2 - playerPos.X) / Game.screenWidth) * (width),
                    ((asteroids[i].pos.Y + asteroids[i].tex.Height / 2 - playerPos.Y) / Game.screenHeight) * (height) ));
                rotations[i] = asteroids[i].rot;
            }

            Vector2 difference = playerPos - targetPos;
            difference.Normalize();
            float angle = difference.ToRadians();

            // Snow is a super thing
            this.target = Snow.rotate(angle + nineteendegrees, asteroidDrawPos(new Vector2(28, 28)), asteroidDrawPos(Vector2.Zero));
        }

        public void draw(SpriteBatch sb)
        {
            sb.Begin();
            sb.Draw(tex, pos, Color.White);
            
            // DRAW ALL THE COMETS!
            for (int i = 0; i < positions.Length; i++) sb.Draw(dot, positions[i], null, asteroidColor, rotations[i],
                new Vector2(dotWidth2, dotWidth2), 1f, SpriteEffects.None, 0);

            sb.Draw(dot, asteroidDrawPos(Vector2.Zero), null, playerColor, playerRot, new Vector2(dotWidth2, dotWidth2), 1f, SpriteEffects.None, 0f);
            sb.Draw(dot, target, Color.White);
            sb.End();
        }

        public Vector2 drawPos(Vector2 pos)
        {
            return new Vector2(pos.X + Game.screenWidth - width - offset - dot.Width / 2, pos.Y + Game.screenHeight - height - offset - dot.Height / 2);
        }

        public Vector2 asteroidDrawPos(Vector2 pos)
        {
            return new Vector2(pos.X + Game.screenWidth - width / 2 - offset - dot.Width / 2, pos.Y + Game.screenHeight - height / 2 - offset - dot.Height / 2);
        }
    }

    // Couldn't come up with another name
    public static class Vector2RadHelper
    {
        public static float ToRadians(this Vector2 pos)
        {
            return (float)Math.Atan2(pos.X, -pos.Y);
        }
    }
}
