using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace Stars
{
    // Man... Porting this was really hard!
    class Game
    {
        Main game;

        StarSystem stars;
        Camera cam;
        Player player;
        PlayerTarget target;
        AsteroidMiniMap minimap;
        FlashingSprite alertScreen;
        MessageBox resetMb;

        AsteroidSystem sys = new AsteroidSystem();

        
        public static bool collisions = true;
        public static float rotateSpeed = 0.001f;
        public const byte asteroidsInImage = 5;

        // Simple solution!
        public static int screenWidth = Main.screenWidth;
        public static int screenHeight = Main.screenHeight;
        public static bool fullscreen = Main.fullscreen;

        public static Rectangle map = new Rectangle(0, 0, Main.screenWidth * 2, Main.screenHeight * 2);

        public float FPS;
        public bool showHUD = true;
        public bool debugData = false;
        public bool pauseMenu = false;
        public bool highscoreMenu = false;
        KeyPressed f1 = new KeyPressed(Keys.F1);
        KeyPressed f2 = new KeyPressed(Keys.F2);
        KeyPressed f3 = new KeyPressed(Keys.F3);
        KeyPressed pauseKey = new KeyPressed(Keys.Escape);
        KeyPressed resetKey = new KeyPressed(Keys.Delete);
        KeyPressed themeKey = new KeyPressed(Keys.F5);

        private int totalFrames;
        private float elapsed;

        public ulong score = 0;

        // Contents
        SpriteFont font;

        // Init stuff
        int sysLength = 45;
        int starsLength = 200;
        int theme = -1;

        public Game(Main g)
        {
            this.game = g;
            Initialize();
        }

        public void reset()
        {
            pauseMenu = false;
            highscoreMenu = false;
            score = 0;
            theme = -1;
            Initialize();
            LoadContent();
        }

        public void Initialize()
        {
            if (Main.configFile) init(game.config);

            game.graphics.PreferredBackBufferWidth = screenWidth;
            game.graphics.PreferredBackBufferHeight = screenHeight;
            game.graphics.IsFullScreen = fullscreen;
            game.graphics.ApplyChanges();
        }

        void init(JConfig config)
        {
            config.get("asteroids", 45, ref sysLength);
            config.get("scene-stars", 200, ref starsLength);
            config.get("scene-theme", -1, ref theme);
            config.get("star-orbit-rotate-speed", 0.001f, ref rotateSpeed);
            config.get("showHUD", true, ref showHUD);
            config.get("debug-data", false, ref debugData);
            config.get("collisions", true, ref collisions);

            string m = "";
            config.get("map", "small", ref m);

            switch (m.ToLower())
            {
                case "medium": setMap(4); break;
                case "large": setMap(8); break;
                case "ultra": setMap(16); break;
                default: setMap(2); break;
            }
        }

        public void setMap(int multiplier)
        {
            map = new Rectangle(0, 0, screenWidth * multiplier, screenWidth * multiplier);
            if (sysLength == 45) sysLength *= multiplier;
        }

        public void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            //spriteBatch = new SpriteBatch(game.GraphicsDevice);
            
            // Thougth let's try that. It actually worked. Wow!
            // (new Random(Guid.NewGuid().GetHashCode()).Next(game.stheme.Length))
            Random ran = new Random(Guid.NewGuid().GetHashCode());

            this.font = game.font;

            if (theme < 0) theme = ran.Next(game.stheme.Length);
            stars = new StarSystem(emptyTexture(5, 5, Color.White, game.GraphicsDevice), game.stheme[theme], new Vector2(ran.Next(20, screenWidth - 20), 
                ran.Next(20, screenHeight - 20)), starsLength);
            sys.init(game.Content.Load<Texture2D>("asteroid"), sysLength);
            cam = new Camera(game.GraphicsDevice.Viewport, map.Width, map.Height);
            minimap = new AsteroidMiniMap(game.GraphicsDevice);
            player = new Player(game.Content.Load<Texture2D>("rocket"), Vector2.Zero, null);
            target = new PlayerTarget(emptyTexture(50, 50, Color.RoyalBlue, game.GraphicsDevice));
            alertScreen = new FlashingSprite(game.GraphicsDevice, Color.Red, screenWidth, screenHeight, Vector2.Zero, 10);

            resetMb = new MessageBox(game.GraphicsDevice, font, Color.Turquoise, "Reset level and start over?", game.mouseTex, true);
        }

        public void Update(GameTime gameTime)
        {
            elapsed += (float)gameTime.ElapsedGameTime.Milliseconds;
            if (elapsed >= 1000f)
            {
                FPS = totalFrames;
                totalFrames = 0;
                elapsed = 0f;
            }

            updateInput();

            player.update((float)gameTime.ElapsedGameTime.TotalSeconds, sys.getNearestAsteroids(player.pos, 50));
            cam.update(player.pos);
            stars.update();
            sys.update();

            if (showHUD)
            {
                // I actually only NEEDED the positions, but since it's called "AsteroidMiniMap"...
                minimap.update(sys.getNearestAsteroids(player.pos, AsteroidMiniMap.width * 2), player.pos, target.pos, player.rot);
            }

            target.update(player);

            if (player.isInDanger) alertScreen.update();
            if (target.acomplished)
            {
                highscoreMenu = true;
            }
            else score += (ulong)gameTime.ElapsedGameTime.TotalMilliseconds;

            byte code = resetMb.update();
            if (code == 1) reset();
        }

        void updateInput()
        {
            KeyboardState state = Keyboard.GetState();

            pauseMenu = pauseKey.pressed(state.IsKeyDown(pauseKey.key));

            if (f1.pressed(state.IsKeyDown(f1.key)))
            {
                if (showHUD) showHUD = false;
                else showHUD = true;
            }

            if (f2.pressed(state.IsKeyDown(f2.key)))
            {
                if (collisions) collisions = false;
                else collisions = true;
            }

            if (f3.pressed(state.IsKeyDown(f3.key)))
            {
                if (debugData) debugData = false;
                else debugData = true;
            }

            if (themeKey.pressed(state.IsKeyDown(themeKey.key)))
            {
                Random ran = new Random(Guid.NewGuid().GetHashCode());
                stars.changeColors(game.stheme[ran.Next(game.stheme.Length)]);
            }

            if (resetKey.pressed(state.IsKeyDown(resetKey.key)))
            {
                resetMb.show = true;
            }
        }

        public void Draw(GameTime gameTime)
        {
            SpriteBatch sb = game.spriteBatch;

            sb.Begin();
            stars.draw(sb);
            sb.End();

            sb.Begin(SpriteSortMode.Deferred, BlendState.AlphaBlend, null, null, null, null, cam.transform);
            sys.draw(sb);
            target.draw(sb);
            player.draw(sb);
            sb.End();

            // HUD last
            if (showHUD)
            {
                minimap.draw(sb);
            }

            // MessageBox
            resetMb.draw(sb);

            if (player.isInDanger)
            {
                sb.Begin();
                alertScreen.draw(sb);
                sb.End();
            }

            // And the debug stuff!
            if (debugData)
            {
                sb.Begin();
                sb.DrawString(font, FPS.ToString() + " FPS", new Vector2(screenWidth - (FPS.ToString().Length * 45) - 20, 20), Color.Red);
                sb.DrawString(font, "Pos: " + player.pos, new Vector2(20, 20), Color.White);
                sb.DrawString(font, "Vel: " + player.vel, new Vector2(20, 40), Color.White);
                sb.DrawString(font, "Rot: " + player.rot + " Radians", new Vector2(20, 60), Color.White);
                sb.DrawString(font, "HUD: " + showHUD, new Vector2(20, 100), Color.White);
                sb.DrawString(font, "Collisions: " + collisions, new Vector2(20, 120), Color.White);
                sb.DrawString(font, "Won: " + target.acomplished, new Vector2(20, 140), Color.White);
                sb.End();
            }

            // Update the total frames for the FPS counter
            totalFrames++;
        }

        public static Texture2D emptyTexture(int width, int height, Color color, GraphicsDevice dev)
        {
            Texture2D tex = new Texture2D(dev, width, height);
            Color[] clr = new Color[width * height];
            for (int i = 0; i < clr.Length; i++)
            {
                clr[i] = color;
            }
            tex.SetData(clr);
            return tex;
        }

        public static Color decreaseAllChannels(Color clr, byte howMany)
        {
            clr.R -= howMany;
            clr.G -= howMany;
            clr.B -= howMany;

            return clr;
        }
    }

    class KeyPressed
    {
        public Keys key;
        private bool keyPressed = false;

        public KeyPressed(Keys key = Keys.A)
        {
            this.key = key;
        }

        public bool pressed(bool what)
        {
            if (what && !keyPressed) { keyPressed = true; return true; }
            else if (!what) keyPressed = false;
            
            return false;
        }
    }

    class MousePressed
    {
        private bool keyPressed = false;

        public bool pressed(bool what)
        {
            if (what) { keyPressed = true; return false; }
            if (keyPressed && !what) { keyPressed = false; return true; } // reset and return true

            return false;
        }
    }
}
