using System;
using System.IO;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;


namespace Stars
{
    class Main : Microsoft.Xna.Framework.Game
    {
        public enum GS
        {
            MainMenu, Playing, PauseMenu, HighscoreMenu
        }

        public GraphicsDeviceManager graphics;
        public SpriteBatch spriteBatch;
        public JConfig config;
        public Log log;
        public Color[] stheme = new Color[] { Color.PowderBlue, // Old mainmenu
            new Color(210, 100, 100), // Old gameplay
            Color.White, new Color(0, 200, 200), new Color(255, 34, 80), Color.White, new Color(255, 255, 0), new Color(200, 0, 255) };

        Game game;
        MainMenu mainMenu;
        PauseMenu pauseMenu;
        HighScoreMenu highscoreMenu;
        GS gameState = GS.MainMenu;

        public const bool configFile = true;
        public static int screenWidth = 800;
        public static int screenHeight = 480;
        public static bool fullscreen = false;
        public bool skipMenu = false;

        public string path;

        // Contents
        public Texture2D mouseTex;
        public SpriteFont font;

        // Mouse
        public Vector2 mousePos;

        // Div
        RenderTarget2D renderTarget;

        // Keys
        KeyPressed scrshotKey = new KeyPressed(Keys.F4);
        


        public Main()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
            path = System.IO.Directory.GetCurrentDirectory();
            
            if (configFile) initFile();

            if (skipMenu) gameState = GS.Playing;

            game = new Game(this);
        }

        protected override void Initialize()
        {
            game.Initialize();

            base.Initialize();
        }

        void initFile()
        {
            config = new JConfig(Content.RootDirectory + "\\settings.txt",
                new string[] { String.Format("// Stars.exe config file made at {0}", DateTime.Now), "// Comments are marked with a \"//\".",
                    "// Everything is case-sentisive."});

            config.get("fullscreen", false, ref fullscreen);
            config.get("width", 800, ref screenWidth);
            config.get("height", 480, ref screenHeight);
            config.get("skip-menu", false, ref skipMenu);
        }

        protected override void LoadContent()
        {
            spriteBatch = new SpriteBatch(GraphicsDevice);

            mouseTex = Content.Load<Texture2D>("menu\\cursor");
            font = Content.Load<SpriteFont>("div\\mainFont");

            Random ran = new Random();
            mainMenu = new MainMenu(GraphicsDevice, font, stheme[ran.Next(stheme.Length)]);
            pauseMenu = new PauseMenu(GraphicsDevice, font);
            highscoreMenu = new HighScoreMenu(GraphicsDevice, Content.RootDirectory);
            log = new Log(Color.Red, new Vector2(20, screenHeight - 40), font);

            game.LoadContent();

            // Run everything once to init stuff
            GameTime gt = new GameTime();
            game.Update(gt);
            mainMenu.update(Vector2.Zero, false);

            renderTarget = new RenderTarget2D(GraphicsDevice, screenWidth, screenHeight);
        }
        

        protected override void Update(GameTime gameTime)
        {
            if (gameState != GS.Playing)
            {
                MouseState mouse = Mouse.GetState();
                mousePos = new Vector2(mouse.X, mouse.Y);
                bool isDown = (mouse.LeftButton == ButtonState.Pressed);

                if (gameState == GS.MainMenu)
                {
                    if (mainMenu.update(mousePos, isDown)) gameState = GS.Playing; // update and check
                }
                else if (gameState == GS.PauseMenu)
                {
                    switch (pauseMenu.update(mousePos, isDown))
                    {
                        case 1: gameState = GS.Playing; break;
                        case 2: gameState = GS.MainMenu; break;
                        case 3: this.Exit(); break;
                        case 4: highscoreMenu.comefrom = gameState; gameState = GS.HighscoreMenu; break;
                        // Break down in tears default
                    }
                }
                else if (gameState == GS.HighscoreMenu)
                {
                    if (highscoreMenu.update(isDown)) gameState = highscoreMenu.comefrom;
                }
            }
            else
            {
                game.Update(gameTime);
                if (game.pauseMenu) gameState = GS.PauseMenu;
                if (game.highscoreMenu)
                {
                    highscoreMenu.justView = false;
                    highscoreMenu.score = game.score;
                    highscoreMenu.comefrom = GS.Playing;
                    gameState = GS.HighscoreMenu;
                    game.reset();
                }
            }

            updateInput();
            log.update(gameTime);

            base.Update(gameTime);
        }

        void updateInput()
        {
            KeyboardState state = Keyboard.GetState();
            
            //if (state.IsKeyDown(Keys.Escape)) this.Exit();

            if (scrshotKey.pressed(state.IsKeyDown(scrshotKey.key))) takeScreenShot();
        }

        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(new Color(1, 3, 0));

            if (gameState != GS.Playing)
            {
                if (gameState == GS.MainMenu) mainMenu.draw(spriteBatch);
                else if (gameState == GS.PauseMenu)
                {
                    game.Draw(gameTime);
                    pauseMenu.draw(spriteBatch);
                }
                else if (gameState == GS.HighscoreMenu)
                {
                    game.Draw(gameTime);
                    highscoreMenu.draw(spriteBatch, font);
                }

                spriteBatch.Begin();
                //spriteBatch.Draw(mouseTex, mousePos, null, Color.White, 0f, Vector2.Zero, 2f, SpriteEffects.None, 0);
                spriteBatch.Draw(mouseTex, mousePos, Color.White);
                spriteBatch.DrawString(font, mousePos.ToString(), new Vector2(20, 20), Color.White);
                spriteBatch.End();
            }
            else game.Draw(gameTime);

            // Draw other stuff
            spriteBatch.Begin();
            log.draw(spriteBatch);
            spriteBatch.End();

            base.Draw(gameTime);
        }

        public void takeScreenShot()
        {
            GraphicsDevice.SetRenderTarget(renderTarget);
            Draw(new GameTime()); // old gametime doesn't work
            GraphicsDevice.SetRenderTarget(null);

            string name = "screenshot_" + Program.getFileTime() + ".png";
            string partPath = path + "\\" + Content.RootDirectory + "\\screenshots";
            string p = partPath + "\\" + name;

            if (!File.Exists(p))
            {
                if (!Directory.Exists(partPath)) Directory.CreateDirectory(partPath);
                using (File.Create(p)) { }
            }

            Stream s = File.OpenWrite(p);
            renderTarget.SaveAsPng(s, screenWidth, screenHeight);
            s.Dispose();

            log.log("Screenshot saved as \"" + name + "\".");
        }
    }
}
