﻿using System;
using JDB;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace Stars
{
    class MainMenu
    {
        Button play;
        Button exit;
        StarSystem sys;

        public MainMenu(GraphicsDevice dev, SpriteFont font, Color clr)
        {
            Random ran = new Random(Guid.NewGuid().GetHashCode());
            play = new Button(dev, new Vector2(Main.screenWidth / 2, Main.screenHeight / 2), font, "Play!", Color.MediumVioletRed);
            exit = new Button(dev, new Vector2(Main.screenWidth / 2, play.pos.Y + 32), font, "Exit now!", Color.Aquamarine);
            sys = new StarSystem(Game.emptyTexture(5, 5, Color.White, dev), clr, new Vector2(ran.Next(20, Game.screenWidth - 20), ran.Next(20, Game.screenHeight - 20)), 200);
        }

        public bool update(Vector2 mousePos, bool down)
        {
            sys.update();
            if (exit.update(mousePos, down)) Environment.Exit(0);
            return play.update(mousePos, down);
        }

        public void draw(SpriteBatch sb)
        {
            sb.Begin();
            sys.draw(sb);
            play.draw(sb);
            exit.draw(sb);
            sb.End();
        }
    }

    class HighScoreMenu
    {
        const byte limit = 10;
        const ushort offset = 100;

        public bool justView = true;
        public ulong score = 0;
        public Main.GS comefrom;
        MousePressed mp = new MousePressed();
        TextBox tb;
        DataBase db;

        public HighScoreMenu(GraphicsDevice dev, string contentDir)
        {
            tb = new TextBox(new Vector2(offset, Game.screenHeight - 60), dev, Color.DarkBlue, 510);
            db = new DataBase(contentDir + "\\scores.txt", new string[] { "Name", "Score" }, ',');
        }

        public void add(string who, ulong value)
        {
            if (db.Rows.Length <= 0) db.add(new string[] { who, value.ToString() });
            else for (int i = 0; i < 10; i++)
            {
                if (i < db.Rows.Length)
                {
                    try
                    {
                        ulong score = Convert.ToUInt64(db.Rows[i][1]);
                        if (score > value)
                        {
                            db.add(new string[] { who, value.ToString() }, i, false);
                            if (db.Rows.Length > limit) db.remove(limit);
                            break;
                        }
                    }
                    catch
                    {
                        db.remove(i);
                    }
                }
                else { db.add(new string[] { who, value.ToString() }); break; }
            }
        }

        public bool update(bool mouseDown)
        {

            if (!justView)
            {
                if (tb.update(25))
                {
                    if (tb.text == "CHUCKNORRIS") score = 0;
                    add(tb.text, score);
                    tb.reset();
                    justView = true;
                }
            }
            else if (mp.pressed(mouseDown)) return true;
            return false;
        }

        public void draw(SpriteBatch sb, SpriteFont font)
        {
            sb.Begin();

            for (int i = 0; i < db.Rows.Length; i++)
            {
                int y = (i + 2) * 21 + 10;
                sb.DrawString(font, (i + 1) + ":", new Vector2(offset, y), Color.Red);
                sb.DrawString(font, db.Rows[i][0], new Vector2(offset + 40, y), Color.Goldenrod);
                sb.DrawString(font, db.Rows[i][1], new Vector2(offset + 410, y), Color.Yellow);
            }

            if (!justView)
            {
                sb.DrawString(font, "You managed a score of " + score + "ms!", new Vector2(offset, Game.screenHeight - 85), Color.White);
                tb.draw(sb, font);
            }

            sb.End();
        }
    }

    class PauseMenu
    {
        KeyPressed exit = new KeyPressed(Keys.Escape);
        KeyPressed back = new KeyPressed(Keys.Back);
        Texture2D background;
        StarSystem sys;
        MenuPart menu;
        Button resume;
        Button tomenu;
        Button quit;
        Button highscores;

        Color menuColor = new Color(128, 0, 0, 100);

        public PauseMenu(GraphicsDevice dev, SpriteFont font)
        {
            Random ran = new Random();
            background = Game.emptyTexture(Game.screenWidth, Game.screenHeight, new Color(0, 0, 0, 180), dev);
            menu = new MenuPart(Game.emptyTexture(160, 220, Color.White, dev), new Vector2(Game.screenWidth / 2, Game.screenHeight / 2));
            sys = new StarSystem(Game.emptyTexture(5, 5, Color.White, dev), Color.White, new Vector2(Game.screenWidth / 2, Game.screenHeight / 2), 120);
            resume = new Button(dev, menu.relPos(new Vector2(10, 10)), font, "Resume", Color.Green, false, 140);
            tomenu = new Button(dev, menu.relPos(new Vector2(10, 40)), font, "Back to menu", Color.Green, false, 140);
            quit = new Button(dev, menu.relPos(new Vector2(10, 100)), font, "Quit", Color.Green, false, 140);
            highscores = new Button(dev, menu.relPos(new Vector2(10, 70)), font, "Highscores", Color.Green, false, 140);
            for (int i = 0; i < sys.snow.Length; i++)
            {
                sys.snow[i].clr = Color.White;
                sys.snow[i].pos = new Vector2(
                    ran.Next(Game.screenWidth / 6, Game.screenWidth - Game.screenWidth / 6),
                    ran.Next(Game.screenHeight / 6, Game.screenHeight - Game.screenHeight / 6));
                // Weird. Forgot to add a ran = new Random(ran.Next(int.MaxValue));, but this does the job too, they don't have all the same position. STRANGE!
            }
            
            // trick the keys
            ret(0);

            sys.update(); // Update at least once to prevent blur
        }

        // I codes sooo much!
        public byte update(Vector2 mousePos, bool down)
        {
            KeyboardState s = Keyboard.GetState();

            if (exit.pressed(s.IsKeyDown(exit.key))) return 3;
            if (back.pressed(s.IsKeyDown(back.key))) return ret(1);

            sys.update();
            
            // trick keys when needed
            if (resume.update(mousePos, down)) return ret(1);
            if (tomenu.update(mousePos, down)) return ret(2);
            if (quit.update(mousePos, down)) return 3;
            if (highscores.update(mousePos, down)) return ret(4);
            return 0;
        }

        private byte ret(byte code)
        {
            back.pressed(true);
            exit.pressed(true);
            return code;
        }

        public void draw(SpriteBatch sb)
        {
            sb.Begin();
            sb.Draw(background, Vector2.Zero, Color.White);
            sys.draw(sb);
            sb.Draw(menu.tex, menu.pos, menuColor);
            resume.draw(sb);
            tomenu.draw(sb);
            quit.draw(sb);
            highscores.draw(sb);
            sb.End();
        }
    }

    struct MenuPart
    {
        public Texture2D tex;
        public Vector2 pos;

        public MenuPart(Texture2D texture, Vector2 position)
        {
            this.tex = texture;
            this.pos = new Vector2(position.X - tex.Width / 2, position.Y - tex.Height / 2);
        }

        public Vector2 relPos(Vector2 p)
        {
            return new Vector2(pos.X + p.X, pos.Y + p.Y);
        }

        public void draw(SpriteBatch sb)
        {
            sb.Draw(tex, pos, Color.White);
        }
    }
}
