﻿using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;


namespace Stars
{
    class MessageBox
    {
        int maxSize;
        private string[] text;
        public string Text
        {
            get { return getText(); }
            set { setText(value); }
        }

        public Button ok;
        public Button yes;
        public Button no;
        private bool yn;
        public bool show;

        Texture2D tex;
        Texture2D pointer;
        Vector2 mousePos;
        Vector2 pos;
        SpriteFont font;

        public MessageBox(GraphicsDevice dev, SpriteFont sfont, Color c, string t, Texture2D cursor, bool yesno = false, bool instandShow = false)
        {
            this.font = sfont;
            this.tex = Game.emptyTexture(Game.screenWidth / 3, Game.screenHeight / 3, c, dev);
            this.pointer = cursor;
            this.pos = new Vector2(Game.screenWidth / 2 - tex.Width / 2, Game.screenHeight / 2 - tex.Height / 2);

            this.yn = yesno;
            if (yn)
            {
                this.yes = new Button(dev, relPos(new Vector2(10, tex.Height - 30)), sfont, "yes", Color.LightCoral, false, -1);
                this.no = new Button(dev, relPos(new Vector2(tex.Width - 55, tex.Height - 30)), sfont, "no", Color.LightCoral, false, -1);
            }
            else this.ok = new Button(dev, relPos(new Vector2(tex.Width / 2, tex.Height - 30)), sfont, "ok", Color.LightCoral, true);
            
            
            this.maxSize = (tex.Width - 10) / 14;
            this.setText(t);
            this.show = instandShow;
        }

        public string getText()
        {
            string toRet = "";
            for (int i = 0; i < text.Length - 1; i++) toRet += text[i] + " ";
            if (text.Length > 0) toRet += text[text.Length - 1];
            return toRet;
        }

        public void setText(string t)
        {
            string[] words = t.Split(' ');
            text = new string[words.Length];

            int curLength = 0;
            int a = 0;

            for (int i = 0; i < words.Length; i++)
            {
                if (curLength + words[i].Length <= maxSize)
                {
                    text[a] += words[i] + " ";
                    curLength += words[i].Length;
                }
                else
                {
                    text[a] = text[a].Remove(text[a].Length - 1);
                    a++;
                    i--; // a++... i-- AI!
                    curLength = 0;
                }
            }
            Array.Resize(ref text, a + 1);
        }

        /// <summary>
        /// UPDATE DA SHIZ!
        /// 0 = nothing happened
        /// 1 = OK / YES
        /// 2 = NO
        /// </summary>
        /// <returns>byte</returns>
        public byte update()
        {
            if (show)
            {
                MouseState mouse = Mouse.GetState();
                this.mousePos = new Vector2(mouse.X, mouse.Y);
                bool down = (mouse.LeftButton == ButtonState.Pressed);

                if (yn)
                {
                    if (no.update(mousePos, down))
                    {
                        show = false; // Close!
                        return 2; // No first, because I don't know.
                    }
                    else if (yes.update(mousePos, down))
                    {
                        show = false; // CLOSE!
                        return 1;
                    }
                }
                else
                {
                    if (ok.update(mousePos, down))
                    {
                        show = false; // CLOOOOSEEEEEEEEEEEE THE MOTHAFUGGGGGGGGAHHHHHHHHHHHH!
                        return 1;
                    }
                }
            }
            return 0;
        }

        public void draw(SpriteBatch sb)
        {
            if (show)
            {
                sb.Begin();
                sb.Draw(tex, pos, Color.White);
                if (yn)
                {
                    yes.draw(sb);
                    no.draw(sb);
                }
                else ok.draw(sb);

                for (int i = 0; i < text.Length; i++)
                {
                    sb.DrawString(font, text[i], relPos(new Vector2(10, i * 15 + 10)), Color.White);
                }
                sb.Draw(pointer, mousePos, Color.White);
                sb.End();
            }
        }

        public Vector2 relPos(Vector2 p)
        {
            return new Vector2(this.pos.X + p.X, this.pos.Y + p.Y);
        }
    }
}
