﻿using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace Stars
{
    class Player
    {
        public Texture2D tex;
        public Vector2 pos;
        public Vector2 vel;
        public Texture2D[] flames;
        
        private KeyPressed resetKey = new KeyPressed(Keys.R);

        public float rot;
        public bool isInDanger = false;

        public Player(Texture2D texture, Vector2 position, Texture2D[] flame)
        {
            this.tex = texture;
            this.pos = position;
            this.flames = flame;
        }

        public void update(float elasped, Asteroid[] asteroids)
        {
            KeyboardState state = Keyboard.GetState();

            // Rotation
            if (state.IsKeyDown(Keys.Left)) rot -= 0.05f;
            else if (state.IsKeyDown(Keys.Right)) rot += 0.05f;


            // Velocity (can you find it?)
            if (state.IsKeyDown(Keys.Up))
            {
                vel.X += elasped * (float)Math.Cos(rot);
                vel.Y += elasped * (float)Math.Sin(rot);
            }

            // Reset velocity
            if (resetKey.pressed(state.IsKeyDown(resetKey.key)))
            {
                if (vel == Vector2.Zero) resetPos();
                vel = Vector2.Zero;
            }

            if (Game.collisions) for (int i = 0; i < asteroids.Length; i++)
            {
                if (asteroids[i].collision(pos, tex)) resetPos();
            }

            isInDanger = (pos.X < 0 - tex.Width || pos.X - tex.Width > Game.map.Width || pos.Y < 0 - tex.Height || pos.Y - tex.Height > Game.map.Height);

            pos += vel;
        }

        public void resetPos()
        {
            this.pos = Vector2.Zero;
            this.rot = 0f;
            this.vel = Vector2.Zero;
        }

        public void draw(SpriteBatch sb)
        {
            sb.Draw(tex, pos, null, Color.White, rot, new Vector2(tex.Width / 2, tex.Height / 2), 1f, SpriteEffects.None, 0f);
        }
    }
}
