﻿using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace Stars
{
    class TextBox
    {
        KeyPressed backspace = new KeyPressed(Keys.Back);
        Keys[] keysPressed;

        Texture2D tex;
        public Vector2 pos;
        public string text = "";

        public TextBox(Vector2 position, GraphicsDevice dev, Color clr, int width)
        {
            this.pos = position;
            this.tex = Game.emptyTexture(width, 26, clr, dev);
        }

        public void reset()
        {
            this.text = "";
            this.keysPressed = null;
        }

        public bool update(int limitLength)
        {
            KeyboardState s = Keyboard.GetState();
            Keys[] keys = s.GetPressedKeys();
            Keys[] newKeys = new Keys[keys.Length];
            int a = 0;

            bool back = backspace.pressed(s.IsKeyDown(backspace.key));

            foreach (Keys k in keys)
            {
                if (k != Keys.None)
                {
                    switch (k)
                    {
                        case Keys.Back: if (back && text.Length > 0) text = text.Remove(text.Length - 1); break;
                        case Keys.Delete: text = ""; break;
                        case Keys.Enter: return true;
                        default:
                            newKeys[a] = k;
                            a++;

                            if (!keysPressed.contains(k) && text.Length < limitLength)
                            {
                                if (k == Keys.Space) text += " ";
                                else text += k.ToString();
                            }
                        break;
                    }
                }
            }

            Array.Resize(ref newKeys, a);
            keysPressed = newKeys;
            return false;
        }

        public void draw(SpriteBatch sb, SpriteFont font)
        {
            sb.Draw(tex, pos, Color.White);
            sb.DrawString(font, text, new Vector2(pos.X + 2, pos.Y + 2), Color.White);
        }
    }

    static class TextBoxHelper
    {
        public static bool contains(this Keys[] keys, Keys curKey)
        {
            if (keys != null) for (int i = 0; i < keys.Length; i++) if (keys[i] == curKey) return true;
            return false;
        }
    }
}
